@@ -19,6 +19,7 @@
 
 #include <jellyfish/misc.hpp>
 #include <jellyfish/divisor.hpp>
+#include <signal.h>
 
 namespace jellyfish {
   /* A word is whatever aligned type used for atomic operations
@@ -98,6 +99,9 @@ namespace jellyfish {
     uint_t get_key_len() const { return key_len; }
     uint_t get_val_len() const { return val_len; }
     uint_t get_lval_len() const { return lval_len; }
+    word   get_max_val(bool large) const { 
+      return (((uint64_t)1) << (large ? lval_len : val_len)) - 1;
+    }
 
     // Discretize and round down number of entries according to length
     // of a block. Return in blocks the number of blocks.
