@@ -25,7 +25,7 @@
 
 namespace jellyfish {
   namespace raw_hash {
-    static const char *file_type = "JFRHSHDN";
+    static const char *file_type  __attribute__((used)) = "JFRHSHDN";
     struct header {
       char     type[8];
       uint64_t key_len;
@@ -85,15 +85,22 @@ namespace jellyfish {
       typedef typename storage_t::iterator iterator;
 
     private:
-      mapped_file  _file;
-      storage_t   *_ary;
-      bool         _canonical;
+      mapped_file         _file;
+      storage_t          *_ary;
+      bool                _canonical;
+      bool                _cary_bit;
+      SquareBinaryMatrix  hash_matrix;
+      SquareBinaryMatrix  hash_inverse_matrix;
 
     public:
       query(mapped_file &map) : 
-        _file(map), _ary(0), _canonical(false) { init(); }
+        _file(map), _ary(0), _canonical(false), _cary_bit(false) { init(); }
       query(std::string filename) : 
-        _file(filename), _ary(0), _canonical(false) { init(); }
+        _file(filename.c_str()), _ary(0), _canonical(false), _cary_bit(false)
+      { init(); }
+      query(const char* filename) : 
+        _file(filename), _ary(0), _canonical(false), _cary_bit(false)
+      { init(); }
 
       ~query() {
         if(_ary)
@@ -108,8 +115,13 @@ namespace jellyfish {
       size_t get_max_reprobe_offset() const { return _ary->get_max_reprobe_offset(); }
       bool   get_canonical() const { return _canonical; }
       void   set_canonical(bool v) { _canonical = v; }
+      bool   get_cary_bit() const { return _cary_bit; }
+      void   set_cary_bit(bool v) { _cary_bit = v; }
+      SquareBinaryMatrix get_hash_matrix() { return hash_matrix; }
+      SquareBinaryMatrix get_hash_inverse_matrix() { return hash_inverse_matrix; }
       storage_t *get_ary() const { return _ary; }
 
+      iterator get_iterator() const { return iterator_all(); }
       iterator iterator_all() const { return _ary->iterator_all(); }
       iterator iterator_slice(size_t slice_number, size_t number_of_slice) const {
         return _ary->iterator_slice(slice_number, number_of_slice);
@@ -124,9 +136,9 @@ namespace jellyfish {
         bool success;
         if(_canonical) {
           typename storage_t::key_t key2 = parse_dna::reverse_complement(key, get_mer_len());
-          success = _ary->get_val(key2 < key ? key2 : key, res, true);
+          success = _ary->get_val(key2 < key ? key2 : key, res, true, _cary_bit);
         } else
-          success = _ary->get_val(key, res, true);
+          success = _ary->get_val(key, res, true, _cary_bit);
         return success ? res : 0;
       }
 
@@ -164,7 +176,6 @@ namespace jellyfish {
                                << "Invalid key length '" << header->key_len << "'";
         // TODO: Should that be in the file instead?
         // reprobes = jellyfish::quadratic_reprobes;
-        SquareBinaryMatrix hash_matrix, hash_inverse_matrix;
         map += hash_matrix.read(map);
         if((uint_t)hash_matrix.get_size() != header->key_len)
           eraise(ErrorReading) << "'" << _file.path() << "': "
