@@ -3,4 +3,4 @@
 cd tests
 . ./compat.sh
 
-${DIR}/test_all
+${DIR}/test_offsets_key_value
